@@ -73,7 +73,7 @@ static void sbr_neg_odd_64_c(int *x)
 {
     int i;
     for (i = 1; i < 64; i += 2)
-        x[i] = -(unsigned)x[i];
+        x[i] = -x[i];
 }
 
 static void sbr_qmf_pre_shuffle_c(int *z)
@@ -233,14 +233,12 @@ static void sbr_hf_g_filt_c(int (*Y)[2], const int (*X_high)[40][2],
     int64_t accu;
 
     for (m = 0; m < m_max; m++) {
-        if (22 - g_filt[m].exp < 61) {
-            int64_t r = 1LL << (22-g_filt[m].exp);
-            accu = (int64_t)X_high[m][ixh][0] * ((g_filt[m].mant + 0x40)>>7);
-            Y[m][0] = (int)((accu + r) >> (23-g_filt[m].exp));
+        int64_t r = 1LL << (22-g_filt[m].exp);
+        accu = (int64_t)X_high[m][ixh][0] * ((g_filt[m].mant + 0x40)>>7);
+        Y[m][0] = (int)((accu + r) >> (23-g_filt[m].exp));
 
-            accu = (int64_t)X_high[m][ixh][1] * ((g_filt[m].mant + 0x40)>>7);
-            Y[m][1] = (int)((accu + r) >> (23-g_filt[m].exp));
-        }
+        accu = (int64_t)X_high[m][ixh][1] * ((g_filt[m].mant + 0x40)>>7);
+        Y[m][1] = (int)((accu + r) >> (23-g_filt[m].exp));
     }
 }
 
